# Copyright (C) 2021 Fraunhofer Institut IOSB, Fraunhoferstr. 1, D 76131
# Karlsruhe, Germany.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from enum import Enum

import frost_sta_client
import datetime


class DataArrayValue:

    class Property(Enum):
        ID = 1
        PHENOMENON_TIME = 2
        RESULT = 3
        RESULT_TIME = 4
        RESULT_QUALITY = 5
        VALID_TIME = 6
        PARAMETERS = 7
        FEATURE_OF_INTEREST = 8

        def to_string(self):
            if self is DataArrayValue.Property.ID:
                return "id"
            if self is DataArrayValue.Property.PHENOMENON_TIME:
                return "phenomenonTime"
            if self is DataArrayValue.Property.RESULT:
                return "result"
            if self is DataArrayValue.Property.RESULT_TIME:
                return "resultTime"
            if self is DataArrayValue.Property.RESULT_QUALITY:
                return "resultQuality"
            if self is DataArrayValue.Property.VALID_TIME:
                return "validTime"
            if self is DataArrayValue.Property.PARAMETERS:
                return "parameters"
            if self is DataArrayValue.Property.FEATURE_OF_INTEREST:
                return "FeatureOfInterest/id"


    class VisibleProperties:
        def __init__(self, all_values: bool):
            self.id = all_values
            self.phenomenon_time = all_values
            self.result = all_values
            self.result_time = all_values
            self.result_quality = all_values
            self.valid_time = all_values
            self.parameters = all_values
            self.feature_of_interest = all_values

        def __init__(self):
            self = DataArrayValue.VisibleProperties(False)

        def __init__(self, select):
            self.id = DataArrayValue.Property.ID in select
            self.phenomenon_time = DataArrayValue.Property.PHENOMENON_TIME in select
            self.result = DataArrayValue.Property.RESULT in select
            self.result_time = DataArrayValue.Property.RESULT_TIME in select
            self.result_quality = DataArrayValue.Property.RESULT_QUALITY in select
            self.valid_time = DataArrayValue.Property.VALID_TIME in select
            self.parameters = DataArrayValue.Property.PARAMETERS in select
            self.feature_of_interest = DataArrayValue.Property.FEATURE_OF_INTEREST in select

        def get_components(self):
            components = []
            if self.id:
                components.append(DataArrayValue.Property.ID.to_string())
            if self.phenomenon_time:
                components.append(DataArrayValue.Property.PHENOMENON_TIME.to_string())
            if self.result:
                components.append(DataArrayValue.Property.RESULT.to_string())
            if self.result_time:
                components.append(DataArrayValue.Property.RESULT_TIME.to_string())
            if self.result_quality:
                components.append(DataArrayValue.Property.RESULT_QUALITY.to_string())
            if self.valid_time:
                components.append(DataArrayValue.Property.VALID_TIME.to_string())
            if self.parameters:
                components.append(DataArrayValue.Property.PARAMETERS.to_string())
            if self.feature_of_interest:
                components.append(DataArrayValue.Property.FEATURE_OF_INTEREST.to_string())
            return components


        def from_observation(self, o: frost_sta_client.Observation):
            value = []
            if self.id:
                value.append(o.id)
            if self.phenomenon_time:
                if type(o.phenomenon_time) == str:
                    value.append(o.phenomenon_time)
                if type(o.phenomenon_time) == datetime.datetime:
                    value.append(o.phenomenon_time.isoformat())
            if self.result:
                value.append(o.result)
            if self.result_time:
                value.append(o.result_time)
            if self.result_quality:
                value.append(o.result_quality)
            if self.valid_time:
                value.append(o.valid_time)
            if self.parameters:
                value.append(o.parameters)
            if self.feature_of_interest:
                value.append(o.feature_of_interest.id)
            return value


    def __init__(self):
        self.datastream = None
        self.multi_datastream = None
        self.visible_properties = None
        self.data_array = []
        self.components = None
        self.observations = []


    @property
    def datastream(self):
        return self._datastream

    @datastream.setter
    def datastream(self, value):
        self._datastream = value

    @property
    def components(self):
        return self._components

    @components.setter
    def components(self, properties):
        if properties is None:
            self._components = None
            return
        if len(self.data_array) >= 1:
            raise ValueError("Can not change components after adding Observations")
        self.visible_properties = self.VisibleProperties(properties)
        self._components = self.visible_properties.get_components()

    @property
    def data_array(self):
        return self._data_array

    @data_array.setter
    def data_array(self, value):
        self._data_array = value

    def add_observation(self, o):
        self.data_array.append(self.visible_properties.from_observation(o))
        self.observations.append(o)

    def __getstate__(self):
        data = {"Datastream": {
                "@iot.id": self.datastream.id
                },
                "components": self.components,
                "dataArray": self.data_array}
        return data




# Copyright (C) 2021 Fraunhofer Institut IOSB, Fraunhoferstr. 1, D 76131
# Karlsruhe, Germany.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

EntityTypes = {
    'Datastream': {
        'singular': 'Datastream',
        'plural': 'Datastreams',
        'class': 'frost_sta_client.model.datastream.Datastream',
        'relations_list': ['Sensor', 'Thing', 'ObservedProperty', 'Observation']
    },
    'MultiDatastream': {
        'singular': 'MultiDatastream',
        'plural': 'MultiDatastreams',
        'class': 'frost_sta_client.model.multi_datastream.MultiDatastream',
        'relations_list': ['Sensor', 'Thing', 'ObservedProperty', 'Observation']
    },
    'FeatureOfInterest': {
        'singular': 'FeatureOfInterest',
        'plural': 'FeaturesOfInterest',
        'class': 'frost_sta_client.model.feature_of_interest.FeatureOfInterest',
        'relations_list': ['Observation']
    },
    'HistoricalLocation': {
        'singular': 'HistoricalLocation',
        'plural': 'HistoricalLocations',
        'class': 'frost_sta_client.model.historical_location.HistoricalLocation',
        'relations_list': ['Thing', 'Location']
    },
    'Actuator': {
        'singular': 'Actuator',
        'plural': 'Actuators',
        'class': 'frost_sta_client.model.actuator.Actuator',
        'relations_list': ['TaskingCapability']
    },
    'Location': {
        'singular': 'Location',
        'plural': 'Locations',
        'class': 'frost_sta_client.model.location.Location',
        'relations_list': ['Thing', 'HistoricalLocation']
    },
    'Observation': {
        'singular': 'Observation',
        'plural': 'Observations',
        'class': 'frost_sta_client.model.observation.Observation',
        'relations_list': ['FeatureOfInterest', 'Datastream', 'MultiDatastream']
    },
    'Thing': {
        'singular': 'Thing',
        'plural': 'Things',
        'class': 'frost_sta_client.model.thing.Thing',
        'relations_list': ['Datastream', 'MultiDatastream', 'Location', 'HistoricalLocation', 'TaskingCapability']
    },
    'ObservedProperty': {
        'singular': 'ObservedProperty',
        'plural': 'ObservedProperties',
        'class': 'frost_sta_client.model.observedproperty.ObservedProperty',
        'relations_list': ['Datastream', 'MultiDatastream']
    },
    'Sensor': {
        'singular': 'Sensor',
        'plural': 'Sensors',
        'class': 'frost_sta_client.model.sensor.Sensor',
        'relations_list': ['Datastream', 'MultiDatastream']
    },
    'Task': {
        'singular': 'Task',
        'plural': 'Tasks',
        'class': 'frost_sta_client.model.task.Task',
        'relations_list': ['TaskingCapability']
    },
    'TaskingCapability': {
        'singular': 'TaskingCapability',
        'plural': 'TaskingCapabilities',
        'class': 'frost_sta_client.model.tasking_capability.TaskingCapability',
        'relations_list': ['Task', 'Actuator', 'Thing']
    },
    'UnitOfMeasurement': {
        'singular': 'UnitOfMeasurement',
        'plural': 'UnitOfMeasurements',
        'class': 'frost_sta_client.model.ext.unitofmeasurement.UnitOfMeasurement'
    },
    'EntityList': {
        'singular': 'EntityList',
        'plural': 'EntityLists',
        'class': 'frost_sta_client.model.ext.entity_list.EntityList'
    }
}

list_for_class = {}
for key, entity_type in EntityTypes.items():
    list_for_class[entity_type["class"]] = entity_type["plural"]

def get_list_for_class(clazz):
    clazz_name = clazz.__module__ + "." + clazz.__name__
    return list_for_class[clazz_name]



# Copyright (C) 2021 Fraunhofer Institut IOSB, Fraunhoferstr. 1, D 76131
# Karlsruhe, Germany.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

class UnitOfMeasurement:
    def __init__(self,
                 name="",
                 symbol="",
                 definition=""):
        self.name = name
        self.symbol = symbol
        self.definition = definition

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        if isinstance(value, str) or value is None:
            self._name = value
            return
        raise ValueError('name should be of type str!')

    @property
    def symbol(self):
        return self._symbol

    @symbol.setter
    def symbol(self, value):
        if isinstance(value, str) or value is None:
            self._symbol = value
            return
        raise ValueError('symbol should be of type str!')

    @property
    def definition(self):
        return self._definition

    @definition.setter
    def definition(self, value):
        if isinstance(value, str) or value is None:
            self._definition = value
            return
        raise ValueError('definition should be of type str!')

    def __getstate__(self):
        data = {
            'symbol': self._symbol,
            'definition': self._definition,
            'name': self._name
        }
        return data

    def __setstate__(self, state):
        self.symbol = state.get("symbol", None)
        self.definition = state.get("definition", None)
        self.name = state.get("name", None)
